local B, L, T = unpack(select(2, ...))


B.I = {['bank'] = {['num'] = 0, ['rows'] = 0},
       ['character'] = {['num'] = 0, ['rows'] = 0}}

--[[//////////////////////////////////////////////////

    MAIN INTERFACE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Main = {}
    function B.CreateFrames()

        ----- Button bar with search
            B.I.Main.Frame.Create()

        ----- bank frame
            B.I.Main.Bank.Create()

        ------ reagent frame
            B.I.Main.Reagent.Create()

        ------ character frame
            B.I.Main.Character.Create()
            B.I.Main.Character.Equipment.Create()

        ------ void frame
            B.I.Main.Void.Create()

        ------ guild frame
            B.I.Main.Guild.Create()

        ------ search frame
            B.I.Main.Search.Create()
    end

    function B.ValidateBags(slots, parent, location, splitstack)

        local rows, num = location.rows, location.num
        local numslots = (T.ROW * (rows - 1)) + num

        if not slots or slots == numslots then
            return-- Already correct amount of slots
        elseif slots < numslots then -- Hide extra slots
            rows, num = B.I.Grid.Remove(parent, slots, rows, num)
        else -- Add or Show missing slots
            rows, num = B.I.Grid.Add(parent, slots, rows, num, splitstack)
        end

        location.rows, location.num = rows, num
        parent:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        parent:SetWidth(22 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.ValidateBankBags(bank)
        local total, page

        total = 0
        for page = 1, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
            total = total + (bank[page] and bank[page]['slots'] or 0)
        end

        B.ValidateBags(total, T.F.mainframe.bank, B.I.bank, B.I.Main.Bank.SplitStack)
    end

    function B.ValidatePlayerBags(bank)
        local total, page

        total = 0
        for page = 0, NUM_BAG_SLOTS do
            total = total + (bank[page] and bank[page]['slots'] or 0)
        end

        B.ValidateBags(total, T.F.mainframe.character, B.I.character, B.I.Main.Character.SplitStack)
    end

    B.I.Main.Frame = {}
    function B.I.Main.Frame.Create()

        T.F.toggleframe = CreateFrame("Frame", "BWtoggleframe", UIParent)
        T.F.toggleframe:SetAllPoints()

        T.F.mainframe = CreateFrame("Frame", "BWmainframe", T.F.toggleframe)
        T.F.mainframe:SetPoint("CENTER")
        T.F.mainframe:SetHeight(25)
        T.F.mainframe:SetWidth(15 + (32 * 6) + 192) -- options + searchbar
        T.F.mainframe:SetFrameStrata("DIALOG")
        T.F.mainframe.layout = 1

        B.I.Backdrop(T.F.mainframe)
        T.F.mainframe:SetMovable(true)
        T.F.mainframe:SetUserPlaced(true)
        T.F.mainframe:SetClampedToScreen(true)

        T.F.mainframe:SetScript("OnMouseDown", B.I.Main.Frame.MouseDown)
        T.F.mainframe:SetScript("OnMouseUp", B.I.Main.Frame.MouseUp)

        -- Create option button
        B.I.Main.Frame.Options.Create()

        -- Create bank button
        B.I.Main.Frame.Bank.Create()

        -- Create reagent button
        B.I.Main.Frame.Reagent.Create()

        -- Create character button
        B.I.Main.Frame.Character.Create()

        -- Create void button
        B.I.Main.Frame.Void.Create()

        -- Create guild button
        B.I.Main.Frame.Guild.Create()

        -- Search bar
        B.I.Main.Frame.Search.Create()

        -- Create count button
        B.I.Main.Frame.Count.Create()
    end

    function B.I.Main.Frame:MouseUp(...)
        self:StopMovingOrSizing()
    end

    function B.I.Main.Frame:MouseDown(...)
        self:StartMoving()
    end

    function B.I.Main.Frame.Button(label, parent, name)
        local button = CreateFrame("Button", name, parent or T.F.mainframe)

        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetWidth(30)
        button:SetHeight(T.F.mainframe:GetHeight())
        button:SetNormalFontObject("GameFontNormal")
        button:SetText(label)
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8",  edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1.05})
        button:SetBackdropBorderColor(0, 0, 0, 0)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetTexture("Interface\\Buttons\\WHITE8x8")
        highlight:SetHeight(2)
        highlight:SetVertexColor(1, 1, 1, 1)
        button:SetHighlightTexture(highlight)
        B.I.Main.Frame.ButtonHighlight(button, "BOTTOM")

        return button
    end

    function B.I.Main.Frame.OptionsButton(text, parent, name, inherit)

        local button = CreateFrame("Button", name, parent or T.F.mainframe, inherit)
        button:SetHeight(12)
        button:SetWidth(50)

        B.I.Backdrop(button)

        local label = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        label:SetTextColor(1, 1, 1)
        label:SetAllPoints()
        label:SetFont(label:GetFont(), 7)
        label:SetJustifyH("CENTER")
        label:SetJustifyV("CENTER")
        label:SetWordWrap(false)
        label:SetText(text)
        button.label = label

        button:HookScript("OnEnter", function(self) self:SetBackdropBorderColor(1, 1, 1, 1) end)
        button:HookScript("OnLeave", function(self) self:SetBackdropBorderColor(0, 0, 0, 1) end)

        return button
    end

--[[//////////////////////////////////////////////////

    OPTIONS FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Main.Frame.Options = {}
    function B.I.Main.Frame.Options.Create()

        T.F.mainframe.optionbutton = B.I.Main.Frame.Button("|cff7E7E7Eo|r", nil, "BWoptionbutton")
        T.F.mainframe.optionbutton:SetHighlightTexture("")
        T.F.mainframe.optionbutton:SetPoint("TOPLEFT", T.F.mainframe, "TOPLEFT")
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.optionbutton:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.optionbutton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        T.F.mainframe.optionbutton:RegisterForDrag("LeftButton")

        T.F.mainframe.optionbutton:SetScript("OnShow", B.I.Main.Frame.Options.Show)
        T.F.mainframe.optionbutton:SetScript("OnEnter", B.I.Main.Frame.Options.Enter)
        T.F.mainframe.optionbutton:SetScript("OnLeave", B.I.Main.Frame.Options.Leave)
        T.F.mainframe.optionbutton:SetScript("OnDragStart", B.I.Main.Frame.Options.DragStart)
        T.F.mainframe.optionbutton:SetScript("OnDragStop", B.I.Main.Frame.Options.DragStop)
        T.F.mainframe.optionbutton:SetScript("OnClick", B.I.Main.Frame.Options.Click)

        B.I.OptionsDropDown.Create()
        B.I.Main.Frame.Toggle.Create()
    end

    function B.I.Main.Frame.Options:Show()
        B.SetFrameLocation()
    end

    function B.I.Main.Frame.Options:Enter()

        self:SetText("|cffffffffo|r")
        GameTooltip:ClearLines()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
        GameTooltip:SetText(T.ACTIVE_TABLE.player.." ("..string.sub(T.ACTIVE_TABLE.faction, 1, 1)..")")
        GameTooltip:AddLine(" ")
        GameTooltip:AddLine(L[10005], 1, 1, 1) -- drag
        GameTooltip:AddLine(L[10006], 1, 1, 1) -- hide
        GameTooltip:AddLine(L[10007], 1, 1, 1) -- menu
        GameTooltip:Show()
    end

    function B.I.Main.Frame.Options:Click(button)

        if button == "RightButton" then
            T.F.mainframe.togglebutton:Show()
            HideDropDownMenu(1)
        else
            ToggleDropDownMenu(1, nil, BWdropDownMenu, "BWoptionbutton", 0, 0, nil, nil, 2)
        end 
    end

    function B.I.Main.Frame.Options:DragStart()
        self:SetText("|cff0FBCE2o|r")
        T.F.mainframe:StartMoving()
    end

    function B.I.Main.Frame.Options:DragStop()
        self:SetText("|cff7E7E7Eo|r")
        T.F.mainframe:StopMovingOrSizing()
        B.SetFrameLocation()
    end

    function B.I.Main.Frame.Options:Leave()
        self:SetText("|cff7E7E7Eo|r")
        GameTooltip:Hide()
    end

    B.I.Main.Frame.Toggle = {}
    function B.I.Main.Frame.Toggle.Create()

        T.F.mainframe.togglebutton = B.I.Main.Frame.Button("|cffE62C2Co|r", T.F.toggleframe)
        T.F.mainframe.togglebutton:SetHighlightTexture("")
        T.F.mainframe.togglebutton:SetAllPoints(T.F.mainframe.optionbutton)
        T.F.mainframe.togglebutton:SetClampedToScreen(true)
        T.F.mainframe.togglebutton:Hide()
        T.F.mainframe.togglebutton:SetBackdropColor(0.07, 0.07, 0.07, 0.8)
        T.F.mainframe.togglebutton:SetBackdropBorderColor(0.15, 0.15, 0.15, 1)

        T.F.mainframe.togglebutton:SetScript("OnEnter", B.I.Main.Frame.Toggle.Enter)
        T.F.mainframe.togglebutton:SetScript("OnLeave", B.I.Main.Frame.Toggle.Leave)
        T.F.mainframe.togglebutton:SetScript("OnShow", B.I.Main.Frame.Toggle.Show)
        T.F.mainframe.togglebutton:SetScript("OnHide", B.I.Main.Frame.Toggle.Hide)
        T.F.mainframe.togglebutton:SetScript("OnClick", B.I.Main.Frame.Toggle.Click)
    end

    function B.I.Main.Frame.Toggle:Show()
        T.F.mainframe:Hide()
    end

    function B.I.Main.Frame.Toggle:Hide()
        T.F.mainframe:Show()
    end

    function B.I.Main.Frame.Toggle:Click()
        self:Hide()
    end

    function B.I.Main.Frame.Toggle:Enter()
        self:SetText("|cff03C926o|r")
        self:SetBackdropBorderColor(0.01, 0.79, 0.15, 1)
    end

    function B.I.Main.Frame.Toggle:Leave()
        self:SetText("|cffE62C2Co|r")
        self:SetBackdropBorderColor(0.15, 0.15, 0.15, 1)
    end

    B.I.OptionsDropDown = {}
    function B.I.OptionsDropDown.Create()

        BWdropDownMenu = CreateFrame("Frame", "BWdropDownMenu", T.F.mainframe, "UIDropDownMenuTemplate")
        BWdropDownMenu:Hide()
        UIDropDownMenu_SetWidth(BWdropDownMenu, 130, 4)
        BWdropDownMenu.point = "TOPLEFT"
        BWdropDownMenu.relativePoint = "BOTTOMLEFT"
        UIDropDownMenu_Initialize(BWdropDownMenu, B.I.OptionsDropDown.Initialize) --The virtual
    end

    function B.I.OptionsDropDown:Initialize(level)
        --drop down menues can have sub menues. The value of level determines the drop down sub menu tier
        local level = level or 1
        local x, k, v

        if level == 1 then
            local info = UIDropDownMenu_CreateInfo()
            if BWsavedtable then
                for k, v in pairs(BWsavedtable) do
                    if not T.OPTIONS[k] then B.I.OptionsDropDown.Menu(info, k, k, level) end
                end
            end
            if T.ACTIVE_TABLE.player then
                B.I.OptionsDropDown.Button(self, info, L[10011].." "..T.ACTIVE_TABLE.player, "delete", false, level)
            end
        elseif level == 2 then
            local info = UIDropDownMenu_CreateInfo()
            if BWsavedtable then
                for k, v in pairs(BWsavedtable[UIDROPDOWNMENU_MENU_VALUE]) do
                    checked = tostring(T.ACTIVE_TABLE.player.."."..string.sub(T.ACTIVE_TABLE.faction, 1, 1)) == k and true or false
                    player, faction = k:match"([^.]*).(.*)"
                    B.I.OptionsDropDown.Button(self, info, player.." ("..string.sub(faction, 1, 1)..")", UIDROPDOWNMENU_MENU_VALUE.."."..k, checked, level)
                end
            end
        end
    end

    function B.I.OptionsDropDown.Menu(info, label, value, level)

        info.text = label
        info.value = value
        info.hasArrow = true
        info.checked = false

        UIDropDownMenu_AddButton(info, level)
    end

    function B.I.OptionsDropDown.Button(self, info, label, value, checked, level)

        info.text = label
        info.hasArrow = false
        info.value = value
        info.func = B.dropDownMenuItem_OnClick
        info.owner = self:GetParent()
        info.checked = checked

        UIDropDownMenu_AddButton(info, level)
    end

--[[//////////////////////////////////////////////////

    BANK FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    B.I.Main.Frame.Bank = {}
    function B.I.Main.Frame.Bank.Create()

        T.F.mainframe.bankbutton = B.I.Main.Frame.Button("|cffffffff"..L[10001].."|r")
        T.F.mainframe.bankbutton:SetPoint("LEFT", T.F.mainframe.optionbutton, "RIGHT")
        T.F.mainframe.bankbutton:SetBackdropColor(T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a)
        T.F.mainframe.bankbutton:SetScript("OnClick", B.I.Main.Frame.Bank.Click)
    end

    function B.I.Main.Frame.Bank:Click(...)

        T.F.mainframe.reagent:Hide()
        T.F.mainframe.character:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.guild:Hide()
        T.F.mainframe.search:Hide()

        if T.F.mainframe.bank:IsVisible() then
            T.F.mainframe.lastbank = nil
            T.F.mainframe.bank:Hide()
        else
            T.F.mainframe.lastbank = "bank"
            T.F.mainframe.bank:Show()
        end
    end

    B.I.Main.Bank = {}
    function B.I.Main.Bank.Create()

        T.F.mainframe.bank = CreateFrame("Frame", "BWbank", T.F.mainframe)
        T.F.mainframe.bank:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT")
        T.F.mainframe.bank:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        T.F.mainframe.bank:SetFrameStrata("HIGH")
        T.F.mainframe.bank:Hide()

        T.F.mainframe.bank:SetScript("OnShow", function(self) T.F.mainframe.bankbutton:LockHighlight() end)
        T.F.mainframe.bank:SetScript("OnHide", function(self) T.F.mainframe.bankbutton:UnlockHighlight() end)

        B.I.Backdrop(T.F.mainframe.bank)
        -- Create toggle for tooltip
        B.I.Main.Bank.Tooltip.Create()
        -- Create toggle for bags
        B.I.Main.Bank.BagToggle.Create()

        -- Create the bank bags
        local i, lasttab

        T.F.mainframe.bank.bags = CreateFrame("Frame", nil, T.F.mainframe.bank)
        for i = 1, NUM_BANKBAGSLOTS do
            T.F.mainframe.bank.bags[i] = {}
            T.F.mainframe.bank.bags[i].tab = B.I.Tab.Create(T.F.mainframe.bank.bags, "BWbank"..i)
            T.F.mainframe.bank.bags[i].tab.search:SetAllPoints(T.F.mainframe.bank.bags[i].tab) -- since we have permanent borders
            T.F.mainframe.bank.bags[i].tab.index = i
            T.F.mainframe.bank.bags[i].tab:SetScript("OnEnter", function(self)
                if (BWsavedtable and BWsavedtable['bankbagtooltiptoggle']) or not BWsavedtable then
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                    if self.link then GameTooltip:SetHyperlink(self.link) end
                    if self.quality then
                        local r, g, b = GetItemQualityColor(self.quality)
                        GameTooltip:SetBackdropBorderColor(r, g, b)
                    end
                    GameTooltip:Show()
                end

                if #T.BW.items == 0 then
                    B.BankBagSearch(self.index + 4)
                end
            end)
            T.F.mainframe.bank.bags[i].tab:SetScript("OnLeave", function(self) 
                GameTooltip:Hide()
                local x, y = B.GetCursorScaledPosition()
                -- Only hide when cursor exits left or right (with exceptions)
                if #T.BW.items == 0 then
                    if self.index == 1 then -- First bag tab
                        if (x < self:GetLeft()) or (x > self:GetRight()) or (y > self:GetTop()) then
                            B.BankBagSearch(false)
                        end
                    elseif self.index == #T.F.mainframe.bank.bags then -- Last bag tab
                        if (x < self:GetLeft()) or (x > self:GetRight()) or (y < self:GetBottom()) then
                            B.BankBagSearch(false)
                        end
                    elseif (x < self:GetLeft()) or (x > self:GetRight()) then -- Middle bags
                        B.BankBagSearch(false)
                    end
                end
            end)

            if i == 1 then
                T.F.mainframe.bank.bags[i].tab:SetPoint("TOPLEFT", T.F.mainframe.bank, "TOPRIGHT", 0, -1)
            else
                T.F.mainframe.bank.bags[i].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -2)
            end
            lasttab = T.F.mainframe.bank.bags[i].tab
        end
    end

    B.I.Main.Bank.Tooltip = {}
    function B.I.Main.Bank.Tooltip.Create()

        T.F.mainframe.bank.tooltiptoggle = B.I.Main.Frame.OptionsButton("TOOLTIP", T.F.mainframe.bank, nil)
        T.F.mainframe.bank.tooltiptoggle:SetPoint("TOPRIGHT", -10, 0)

        T.F.mainframe.bank.tooltiptoggle:HookScript("OnClick", B.I.Main.Bank.Tooltip.Click)
        T.F.mainframe.bank.tooltiptoggle:HookScript("OnEnter", B.I.Main.Bank.Tooltip.Enter)
        T.F.mainframe.bank.tooltiptoggle:HookScript("OnLeave", B.I.Main.Bank.Tooltip.Leave)
    end

    function B.I.Main.Bank.Tooltip:Click(...)
        BWsavedtable['bankbagtooltiptoggle'] = not BWsavedtable['bankbagtooltiptoggle']
    end

    function B.I.Main.Bank.Tooltip:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.I.Main.Bank.Tooltip:Leave()
        GameTooltip:Hide()
    end

    B.I.Main.Bank.BagToggle = {}
    function B.I.Main.Bank.BagToggle.Create()

        T.F.mainframe.bank.bagstoggle = B.I.Main.Frame.OptionsButton("BAGS", T.F.mainframe.bank, nil)
        T.F.mainframe.bank.bagstoggle:SetPoint("TOPRIGHT", T.F.mainframe.bank.tooltiptoggle, "TOPLEFT", -5, 0)

        T.F.mainframe.bank.bagstoggle:HookScript("OnClick", B.I.Main.Bank.BagToggle.Click)
        T.F.mainframe.bank.bagstoggle:HookScript("OnEnter", B.I.Main.Bank.BagToggle.Enter)
        T.F.mainframe.bank.bagstoggle:HookScript("OnLeave", B.I.Main.Bank.BagToggle.Leave)
    end

    function B.I.Main.Bank.BagToggle:Click(...)

        if T.F.mainframe.bank.bags:IsVisible() then
            T.F.mainframe.bank.bags:Hide()
            BWsavedtable['bankbagstoggle'] = false
        else
            T.F.mainframe.bank.bags:Show()
            BWsavedtable['bankbagstoggle'] = true
        end
    end

    function B.I.Main.Bank.BagToggle:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.I.Main.Bank.BagToggle:Leave()
        GameTooltip:Hide()
    end

    function B.I.Main.Bank.Buttons()

        B.I.Frame.Create(T.F.mainframe.bank, T.F.mainframe.bank)
        B.I.bank.rows, B.I.bank.num = B.I.Grid.Create(T.F.mainframe.bank, NUM_BANKGENERIC_SLOTS, B.I.bank.rows, B.I.bank.num, B.I.Main.Bank.SplitStack)

        T.F.mainframe.bank:SetHeight(10 + (T.BUTTON_HEIGHT * (B.I.bank.rows + 1)) + (T.FRAMEWHITESPACE * B.I.bank.rows))
        T.F.mainframe.bank:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Main.Bank.SplitStack(button, split)
        local bag, slot = B.GetBankBag(button)
        SplitContainerItem(bag, slot, split)
    end

--[[//////////////////////////////////////////////////

    REAGENT FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Main.Frame.Reagent = {}
    function B.I.Main.Frame.Reagent.Create()

        T.F.mainframe.reagentbutton = B.I.Main.Frame.Button("|cffffffff"..L[10002].."|r")
        T.F.mainframe.reagentbutton:SetPoint("LEFT", T.F.mainframe.bankbutton, "RIGHT")
        T.F.mainframe.reagentbutton:SetBackdropColor(T.REAGENTTAB.r, T.REAGENTTAB.g, T.REAGENTTAB.b, T.REAGENTTAB.a)
        T.F.mainframe.reagentbutton:SetScript("OnClick", B.I.Main.Frame.Reagent.Click)
    end

    function B.I.Main.Frame.Reagent:Click(...)

        T.F.mainframe.bank:Hide()
        T.F.mainframe.character:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.guild:Hide()
        T.F.mainframe.search:Hide()

        if T.F.mainframe.reagent:IsVisible() then
            T.F.mainframe.lastbank = nil
            T.F.mainframe.reagent:Hide()
        else
            T.F.mainframe.lastbank = "reagent"
            T.F.mainframe.reagent:Show()
        end
    end

    B.I.Main.Reagent = {}
    function B.I.Main.Reagent.Create()

        T.F.mainframe.reagent = CreateFrame("Frame", "BWreagent", T.F.mainframe)
        T.F.mainframe.reagent:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT")
        T.F.mainframe.reagent:SetFrameStrata("HIGH")
        T.F.mainframe.reagent:Hide()

        T.F.mainframe.reagent:SetScript("OnShow", B.I.Main.Reagent.Show)
        T.F.mainframe.reagent:SetScript("OnHide", B.I.Main.Reagent.Hide)

        B.I.Backdrop(T.F.mainframe.reagent)
    end

    function B.I.Main.Reagent:Show()
        T.F.mainframe.reagentbutton:LockHighlight()
    end

    function B.I.Main.Reagent:Hide()
        T.F.mainframe.reagentbutton:UnlockHighlight()
    end

    function B.I.Main.Reagent.Buttons()
        local x, rows

        for x = 1, T.REAGENT_PAGES do
            T.F.mainframe.reagent[x] = {}
            B.I.Frame.Create(T.F.mainframe.reagent, T.F.mainframe.reagent[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.reagent[x], T.NUM_REAGENT_SLOTS, nil, nil, B.I.Main.Reagent.SplitStack)
        end

        T.F.mainframe.reagent:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.reagent:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Main.Reagent.SplitStack(button, split)
        SplitContainerItem(REAGENTBANK_CONTAINER, button:GetID(), split)
    end

--[[//////////////////////////////////////////////////

    CHARACTER FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    B.I.Main.Frame.Character = {}
    function B.I.Main.Frame.Character.Create()

        T.F.mainframe.characterbutton = B.I.Main.Frame.Button("|cffffffff"..L[10010].."|r")
        T.F.mainframe.characterbutton:SetPoint("LEFT", T.F.mainframe.reagentbutton, "RIGHT")
        T.F.mainframe.characterbutton:SetBackdropColor(T.CHARTAB.r, T.CHARTAB.g, T.CHARTAB.b, T.CHARTAB.a)
        T.F.mainframe.characterbutton:SetScript("OnClick", B.I.Main.Frame.Character.Click)
    end

    function B.I.Main.Frame.Character:Click(...)

        T.F.mainframe.bank:Hide()
        T.F.mainframe.reagent:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.guild:Hide()
        T.F.mainframe.search:Hide()

        if T.F.mainframe.character:IsVisible() then
            T.F.mainframe.lastbank = nil
            T.F.mainframe.character:Hide()
        else
            T.F.mainframe.lastbank = "character"
            T.F.mainframe.character:Show()
        end
    end

    B.I.Main.Character = {}
    function B.I.Main.Character.Create()

        T.F.mainframe.character = CreateFrame("Frame", "BWcharacter", T.F.mainframe)
        T.F.mainframe.character:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT")
        T.F.mainframe.character:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        T.F.mainframe.character:SetFrameStrata("HIGH")
        T.F.mainframe.character:Hide()

        T.F.mainframe.character:SetScript("OnShow", B.I.Main.Character.Show)
        T.F.mainframe.character:SetScript("OnHide", B.I.Main.Character.Hide)

        B.I.Backdrop(T.F.mainframe.character)

        -- Create toggle for tooltip
        B.I.Main.Character.Tooltip.Create()
        -- Create toggle for bags
        B.I.Main.Character.BagToggle.Create()

        T.F.mainframe.character.tab = B.I.Tab.Create(T.F.mainframe.character)
        T.F.mainframe.character.tab:SetPoint("TOPLEFT", T.F.mainframe.character, "TOPRIGHT", 0, -1)
        T.F.mainframe.character.tab:SetNormalTexture("Interface\\PaperDollInfoFrame\\PaperDollSidebarTabs")
        T.F.mainframe.character.tab:GetNormalTexture():SetTexCoord(0.01562500, 0.53125000, 0.46875000, 0.60546875)
        T.F.mainframe.character.tab:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.character.tab:HookScript("OnClick", function(self)
            if T.F.mainframe.character.equipment:IsVisible() then
                self:SetBackdropBorderColor(0, 0, 0, 1)
                T.F.mainframe.character.equipment:Hide()
            else
                self:SetBackdropBorderColor(1, 1, 1, 1)
                T.F.mainframe.character.equipment:Show()
            end
        end)
        T.F.mainframe.character.tab:HookScript("OnEnter", function(self)
            if T.F.mainframe.layout > 2 then
                GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
            else
                GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            end
            GameTooltip:SetText(BAG_FILTER_EQUIPMENT)
            GameTooltip:Show()
        end)
        T.F.mainframe.character.tab:SetScript("OnLeave", function(self) GameTooltip:Hide() end)

        -- Create artifact power for bags
        B.I.Main.Character.Artifact.Create()

        -- Create the bank bags
        local i, lasttab
        
        T.F.mainframe.character.bags = CreateFrame("Frame", nil, T.F.mainframe.character)
        for i = 1, NUM_BAG_SLOTS do
            T.F.mainframe.character.bags[i] = {}
            T.F.mainframe.character.bags[i].tab = B.I.Tab.Create(T.F.mainframe.character.bags, "BWcharacter"..i)
            T.F.mainframe.character.bags[i].tab.search:SetAllPoints(T.F.mainframe.character.bags[i].tab) -- since we have permanent borders
            T.F.mainframe.character.bags[i].tab.index = i


            T.F.mainframe.character.bags[i].tab:SetScript("OnEnter", function(self)
                if (BWsavedtable and BWsavedtable['playerbagtooltiptoggle']) or not BWsavedtable then
                    local r, g, b = GetItemQualityColor(self.quality)
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                    if self.link then GameTooltip:SetHyperlink(self.link) end
                    GameTooltip:SetBackdropBorderColor(r, g, b)
                    GameTooltip:Show()
                end

                if #T.BW.items == 0 then
                    B.PlayerBagSearch(i)
                end
            end)

            T.F.mainframe.character.bags[i].tab:SetScript("OnLeave", function(self) 
                GameTooltip:Hide()
                local x, y = B.GetCursorScaledPosition()
                -- Only hide when cursor exits left or right (with exceptions)
                if #T.BW.items == 0 then
                    if self.index == 1 then -- First bag tab
                        if (x < self:GetLeft()) or (x > self:GetRight()) or (y > self:GetTop()) then
                            B.PlayerBagSearch(false)
                        end
                    elseif self.index == #T.F.mainframe.character.bags then -- Last bag tab
                        if (x < self:GetLeft()) or (x > self:GetRight()) or (y < self:GetBottom()) then
                            B.PlayerBagSearch(false)
                        end
                    elseif (x < self:GetLeft()) or (x > self:GetRight()) then -- Middle bags
                        B.PlayerBagSearch(false)
                    end
                end
            end)

            if i == 1 then
                T.F.mainframe.character.bags[i].tab:SetPoint("TOPLEFT", T.F.mainframe.character.artifact, "BOTTOMLEFT", 0, -2)
            else
                T.F.mainframe.character.bags[i].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -2)
            end
            lasttab = T.F.mainframe.character.bags[i].tab
        end
    end

    function B.I.Main.Character.Buttons()

        B.I.Frame.Create(T.F.mainframe.character, T.F.mainframe.character)
        B.I.character.rows, B.I.character.num = B.I.Grid.Create(T.F.mainframe.character, 30, B.I.character.rows, B.I.character.num, B.I.Main.Character.SplitStack)

        T.F.mainframe.character:SetHeight(10 + (T.BUTTON_HEIGHT * (B.I.character.rows + 1)) + (T.FRAMEWHITESPACE * B.I.character.rows))
        T.F.mainframe.character:SetWidth(22 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Main.Character.SplitStack(button, split)
        local bag, slot = B.GetBag(button)
        SplitContainerItem(bag, slot, split)
    end

    B.I.Main.Character.Tooltip = {}
    function B.I.Main.Character.Tooltip.Create()

        T.F.mainframe.character.tooltiptoggle = B.I.Main.Frame.OptionsButton("TOOLTIP", T.F.mainframe.character, nil)
        T.F.mainframe.character.tooltiptoggle:SetPoint("TOPRIGHT", -10, 0)

        T.F.mainframe.character.tooltiptoggle:HookScript("OnClick", B.I.Main.Character.Tooltip.Click)
        T.F.mainframe.character.tooltiptoggle:HookScript("OnEnter", B.I.Main.Character.Tooltip.Enter)
        T.F.mainframe.character.tooltiptoggle:HookScript("OnLeave", B.I.Main.Character.Tooltip.Leave)
    end

    function B.I.Main.Character.Tooltip:Click(...)
        BWsavedtable['playerbagtooltiptoggle'] = not BWsavedtable['playerbagtooltiptoggle']
    end

    function B.I.Main.Character.Tooltip:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.I.Main.Character.Tooltip:Leave()
        GameTooltip:Hide()
    end

    B.I.Main.Character.BagToggle = {}
    function B.I.Main.Character.BagToggle.Create()

        T.F.mainframe.character.bagstoggle = B.I.Main.Frame.OptionsButton("BAGS", T.F.mainframe.character, nil)
        T.F.mainframe.character.bagstoggle:SetPoint("TOPRIGHT", T.F.mainframe.character.tooltiptoggle, "TOPLEFT", -5, 0)
    
        T.F.mainframe.character.bagstoggle:HookScript("OnClick", B.I.Main.Character.BagToggle.Click)
        T.F.mainframe.character.bagstoggle:HookScript("OnEnter", B.I.Main.Character.BagToggle.Enter)
        T.F.mainframe.character.bagstoggle:HookScript("OnLeave", B.I.Main.Character.BagToggle.Leave)
    end

    function B.I.Main.Character.BagToggle:Click(...)

        if T.F.mainframe.character.bags:IsVisible() then
            T.F.mainframe.character.bags:Hide()
            BWsavedtable['playerbagstoggle'] = false
        else
            T.F.mainframe.character.bags:Show()
            BWsavedtable['playerbagstoggle'] = true
        end
    end

    function B.I.Main.Character.BagToggle:Enter()
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.I.Main.Character.BagToggle:Leave()
        GameTooltip:Hide()
    end

    B.I.Main.Character.Artifact = {}
    function B.I.Main.Character.Artifact.Create()

        T.F.mainframe.character.artifact = B.I.Tab.Create(T.F.mainframe.character, "BWartifact", "SecureActionButtonTemplate")
        T.F.mainframe.character.artifact:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "BOTTOMLEFT", 0, -2)
        T.F.mainframe.character.artifact:SetNormalTexture(463517)
        T.F.mainframe.character.artifact:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.character.artifact:SetAttribute("type", "macro")
        T.F.mainframe.character.artifact:Show() -- To make keybinding work without first displaying

        T.F.mainframe.character.artifact:HookScript("PreClick", B.I.Main.Character.Artifact.PreClick)
        T.F.mainframe.character.artifact:HookScript("OnClick", B.I.Main.Character.Artifact.Click)
        T.F.mainframe.character.artifact:HookScript("OnEnter", B.I.Main.Character.Artifact.Enter)
        T.F.mainframe.character.artifact:HookScript("OnLeave", B.I.Main.Character.Artifact.Leave)
    end

    function B.I.Main.Character.Artifact:PreClick(...)
        local x, page, counter, button, buttons, name
        local player = T['player']

        local counter = 1
        for page = 0, #player do
            if player[page] then
                buttons = T.F.mainframe.character.buttons
                for x = 1, player[page]['slots'] do
                    button = buttons[counter]

                    if button.name then
                        if IsArtifactPowerItem(B.GetItemIdFromLink(button.link)) then
                            self:SetAttribute("macrotext", format("/use %d %d", B.GetBag(button)))
                            B.print("Using: |T"..button.icon:GetTexture()..":0|t"..button.link..": "..B.DetectAP(B.GetBag(button)).." on "..player[-1][16]['link'], 1)
                            return
                        end
                    end
                    counter = counter + 1
                end
            end
        end
    end

    function B.I.Main.Character.Artifact:Click(...)
        self:SetAttribute("macrotext", nil)
    end

    function B.I.Main.Character.Artifact:Enter()
        self.total = nil
        B.PlayerArtifactSearch()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        GameTooltip:SetText(ITEM_QUALITY_COLORS[6].hex..ARTIFACT_POWER.."|r") -- "|cffBD76FF"
        GameTooltip:AddLine(L[10019])
        if self.total then
            GameTooltip:AddLine(" ")
            GameTooltip:AddLine("|cffffffff"..self.total.."|r")
        end
        GameTooltip:Show()
    end

    function B.I.Main.Character.Artifact:Leave()

        text = T.F.mainframe.searchbar:GetText()
        if text and text ~= "" then
            B.PlayerSearch(text)
        else
            B.PlayerSearch(false)
        end

        GameTooltip:Hide()
    end

    function B.I.Main.Character:Show()
        T.F.mainframe.characterbutton:LockHighlight()
    end

    function B.I.Main.Character:Hide()
        T.F.mainframe.characterbutton:UnlockHighlight()
    end

    B.I.Main.Character.Equipment = {}
    function B.I.Main.Character.Equipment.Create()

        T.F.mainframe.character.equipment = CreateFrame("Frame", "BWcharacterequipment", T.F.mainframe.character)
        T.F.mainframe.character.equipment:SetPoint("TOPLEFT", T.F.mainframe.character.tab, "TOPRIGHT")
        T.F.mainframe.character.equipment:SetWidth(20 + (T.BUTTON_HEIGHT * 5) + (T.FRAMEWHITESPACE * (5 - 1)))
        T.F.mainframe.character.equipment:SetFrameStrata("HIGH")
        T.F.mainframe.character.equipment:Hide()

        B.I.Backdrop(T.F.mainframe.character.equipment)

        B.I.Main.Character.Equipment.Buttons()
    end

    function B.I.Main.Character.Equipment.Buttons()
        local x, button, lastbutton, lastrow, rows, num

        B.I.Frame.Create(T.F.mainframe.character.equipment, T.F.mainframe.character.equipment)
        rows, num = 0, 0

        for x = 1, 18 do

            button = B.I.Button.Create(x, T.F.mainframe.character.equipment.frame)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            button:HookScript("OnEnter", function(self)
                if not self.link then
                    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
                    GameTooltip:SetText(_G[T.INVSLOTS[self:GetID()]])
                    GameTooltip:Show()
                end
            end)
            
            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT", 1, 0)
                rows, num = rows + 1, 1
                lastrow = button

            elseif num == 5 then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end
            T.F.mainframe.character.equipment.buttons[x] = button
            lastbutton = button
        end
        T.F.mainframe.character.equipment:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
    end

--[[//////////////////////////////////////////////////

    VOID FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    B.I.Main.Frame.Void = {}
    function B.I.Main.Frame.Void.Create()

        T.F.mainframe.voidbutton = B.I.Main.Frame.Button("|cffffffff"..L[10003].."|r")
        T.F.mainframe.voidbutton:SetPoint("LEFT", T.F.mainframe.characterbutton, "RIGHT")
        T.F.mainframe.voidbutton:SetBackdropColor(T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a)
        T.F.mainframe.voidbutton:SetScript("OnClick", B.I.Main.Frame.Void.Click)
    end

    function B.I.Main.Frame.Void:Click(...)

        T.F.mainframe.bank:Hide()
        T.F.mainframe.reagent:Hide()
        T.F.mainframe.character:Hide()
        T.F.mainframe.guild:Hide()
        T.F.mainframe.search:Hide()

        if T.F.mainframe.void:IsVisible() then
            T.F.mainframe.lastbank = nil
            T.F.mainframe.void:Hide()
        else
            T.F.mainframe.lastbank = "void"
            T.F.mainframe.void:Show()
        end
    end

    B.I.Main.Void = {}
    function B.I.Main.Void.Create()

        T.F.mainframe.void = CreateFrame("Frame", "BWvoid", T.F.mainframe)
        T.F.mainframe.void:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT")
        T.F.mainframe.void:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        T.F.mainframe.void:SetFrameStrata("HIGH")
        T.F.mainframe.void.lasttab = 1
        T.F.mainframe.void:Hide()

        T.F.mainframe.void:SetScript("OnShow", B.I.Main.Void.Click)
        T.F.mainframe.void:SetScript("OnHide", B.I.Main.Void.Hide)

        B.I.Backdrop(T.F.mainframe.void)
        B.I.Main.Void.Tabs.Create()
    end

    function B.I.Main.Void:Show(...)
        T.F.mainframe.voidbutton:LockHighlight()
    end

    function B.I.Main.Void:Hide()
        T.F.mainframe.voidbutton:UnlockHighlight()
    end

    B.I.Main.Void.Tabs = {}
    function B.I.Main.Void.Tabs.Create()
        local x, lasttab

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x] = {}
            T.F.mainframe.void[x].tab = B.I.Tab.Create(T.F.mainframe.void)
            T.F.mainframe.void[x].tab.index = x
            T.F.mainframe.void[x].tab:SetScript("OnClick", B.I.Main.Void.Tabs.Click)

            if x == 1 then
                T.F.mainframe.void[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceCosmicGreater")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", T.F.mainframe.void, "TOPRIGHT", 0, -1)
            else
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceArcaneLarge")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = T.F.mainframe.void[x].tab
        end
    end

    function B.I.Main.Void.Tabs:Click(...)
        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISVOIDOPEN then
            VoidStorage_SetPageNumber(self.index)
        end
    end

    function B.I.Main.Void.Buttons()
        local x, rows

        for x = 1, T.VOID_PAGES do
            B.I.Frame.Create(T.F.mainframe.void, T.F.mainframe.void[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.void[x], T.NUM_VOID_SLOTS)
        end

        T.F.mainframe.void:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.void:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

--[[//////////////////////////////////////////////////

    GUILD FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    B.I.Main.Frame.Guild = {}
    function B.I.Main.Frame.Guild.Create()

        T.F.mainframe.guildbutton = B.I.Main.Frame.Button("|cffffffff"..L[10004].."|r")
        T.F.mainframe.guildbutton:SetPoint("LEFT", T.F.mainframe.voidbutton, "RIGHT")
        T.F.mainframe.guildbutton:SetBackdropColor(T.GUILDTAB.r, T.GUILDTAB.g, T.GUILDTAB.b, T.GUILDTAB.a)

        T.F.mainframe.guildbutton:SetScript("OnClick", B.I.Main.Frame.Guild.Click)
        T.F.mainframe.guildbutton:HookScript("OnEnter", B.I.Main.Frame.Guild.Enter)
        T.F.mainframe.guildbutton:HookScript("OnLeave", B.I.Main.Frame.Guild.Leave)
    end

    function B.I.Main.Frame.Guild:Click(...)

        T.F.mainframe.bank:Hide()
        T.F.mainframe.reagent:Hide()
        T.F.mainframe.character:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.search:Hide()

        if T.F.mainframe.guild:IsVisible() then
            T.F.mainframe.lastbank = nil
            T.F.mainframe.guild:Hide()
        else
            T.F.mainframe.lastbank = "guild"
            T.F.mainframe.guild:Show()
        end
    end

    function B.I.Main.Frame.Guild:Enter()
        local name = B.GetPlayerBank('guild')['name']

        if name then
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            GameTooltip:SetText(name)
            GameTooltip:Show()
        end
    end

    function B.I.Main.Frame.Guild:Leave()
        GameTooltip:Hide()
    end

    B.I.Main.Guild = {}
    function B.I.Main.Guild.Create()

        T.F.mainframe.guild = CreateFrame("Frame", "BWguild", T.F.mainframe)
        T.F.mainframe.guild:SetPoint("TOPLEFT", T.F.mainframe, "BOTTOMLEFT")
        T.F.mainframe.guild:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        T.F.mainframe.guild:SetFrameStrata("HIGH")
        T.F.mainframe.guild.lasttab = 1
        T.F.mainframe.guild:Hide()

        T.F.mainframe.guild:SetScript("OnShow", B.I.Main.Guild.Show)
        T.F.mainframe.guild:SetScript("OnHide", B.I.Main.Guild.Hide)

        B.I.Backdrop(T.F.mainframe.guild)
        B.I.Main.Guild.Tabs.Create()
    end

    function B.I.Main.Guild:Show()
        T.F.mainframe.guildbutton:LockHighlight()
    end

    function B.I.Main.Guild:Hide()
        T.F.mainframe.guildbutton:UnlockHighlight()
    end

    B.I.Main.Guild.Tabs = {}
    function B.I.Main.Guild.Tabs.Create()
        local x, lasttab

        for x = 1, MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x] = {}
            T.F.mainframe.guild[x].tab = B.I.Tab.Create(T.F.mainframe.guild)
            T.F.mainframe.guild[x].tab.index = x
            T.F.mainframe.guild[x].tab:SetScript("OnClick", B.I.Main.Guild.Tabs.Click)
            T.F.mainframe.guild[x].tab:SetScript("OnEnter", B.I.Main.Guild.Tabs.Enter)
            T.F.mainframe.guild[x].tab:SetScript("OnLeave", B.I.Main.Guild.Tabs.Leave)

            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", T.F.mainframe.guild, "TOPRIGHT", 0, -1)
                T.F.mainframe.guild[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = T.F.mainframe.guild[x].tab
        end
    end

    function B.I.Main.Guild.Tabs:Click(...)
        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISGUILDOPEN then
            _G['GuildBankTab'..self.index..'Button']:Click()
        end
    end

    function B.I.Main.Guild.Tabs:Enter()

        if T.F.mainframe.layout > 2 then
            GameTooltip:SetOwner(self, "ANCHOR_LEFT", 10, -10)
        else
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        end
        if self.name then GameTooltip:SetText(self.name) end
        GameTooltip:Show()
    end

    function B.I.Main.Guild.Tabs:Leave()

        GameTooltip:Hide()
    end

    function B.I.Main.Guild.Buttons()
        local x, rows

        for x = 1, MAX_GUILDBANK_TABS do
            B.I.Frame.Create(T.F.mainframe.guild, T.F.mainframe.guild[x], x)
            rows = B.I.Grid.Create(T.F.mainframe.guild[x], T.MAX_GUILDBANK_SLOTS_PER_TAB, nil, nil, B.I.Main.Guild.SplitStack)
        end

        T.F.mainframe.guild:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.guild:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end

    function B.I.Main.Guild.SplitStack(button, split)
        SplitGuildBankItem(T.F.mainframe.guild.lasttab, button:GetID(), split)
    end

--[[//////////////////////////////////////////////////

    SEARCH FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    B.I.Main.Frame.Search = {}
    function B.I.Main.Frame.Search.Create()

        T.F.mainframe.searchbar = CreateFrame("EditBox", "BW_Searchbar", T.F.mainframe, "BagSearchBoxTemplate")
        T.F.mainframe.searchbar:SetHeight(25)
        T.F.mainframe.searchbar:SetWidth(192)
        T.F.mainframe.searchbar:SetAutoFocus(false)
        T.F.mainframe.searchbar:SetPoint("LEFT", T.F.mainframe.guildbutton, "RIGHT", 10, 0)
        T.F.mainframe.searchbar:SetHyperlinksEnabled(true)
        T.F.mainframe.searchbar:SetMultiLine(false)
        T.F.mainframe.searchbar:SetMaxLetters(0)

        T.F.mainframe.searchbar:HookScript("OnEscapePressed", B.I.Main.Frame.Search.EscapePressed)
        T.F.mainframe.searchbar:HookScript("OnTextChanged", B.I.Main.Frame.Search.TextChanged)

        B.I.RemoveEdge(T.F.mainframe.searchbar)
        tinsert(ITEM_SEARCHBAR_LIST, "BW_Searchbar")
    end

    function B.I.Main.Frame.Search:TextChanged(userInput)

        text = self:GetText()
        T.F.mainframe.countbutton.count = 0

        if text == "" or text == SEARCH then T.F.mainframe:SetBackdropColor(0, 0, 0, 0.5) end

        if text == SEARCH or text == "" then
            B.Search(false)
            collectgarbage()
        else
            T.F.mainframe:SetBackdropColor(0.05, 0.05, 0.05, 1)
            B.Search(text)
        end
    end

    function B.I.Main.Frame.Search:EscapePressed()
        self:ClearFocus()
    end

    B.I.Main.Frame.Count = {}
    function B.I.Main.Frame.Count.Create()

        T.F.mainframe.countbutton = B.I.Main.Frame.Button("|cffffffff|r")
        T.F.mainframe.countbutton:SetWidth(64)
        T.F.mainframe.countbutton:SetPoint("LEFT", T.F.mainframe.searchbar, "RIGHT")
        T.F.mainframe.countbutton:SetBackdropColor(0.05, 0.05, 0.05, 1)
        T.F.mainframe.countbutton.count = 0
        T.F.mainframe.countbutton:Hide()

        T.F.mainframe.countbutton:SetScript("OnHide", B.I.Main.Frame.Count.Hide)
        T.F.mainframe.countbutton:SetScript("OnClick", B.I.Main.Frame.Count.Click)

        T.F.mainframe.countbutton.labelcount = T.F.mainframe.countbutton:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        T.F.mainframe.countbutton.labelcount:SetFont(T.F.mainframe.countbutton.labelcount:GetFont(), 14) -- Resize string
        T.F.mainframe.countbutton.labelcount:SetJustifyH("CENTER")
        T.F.mainframe.countbutton.labelcount:SetPoint("CENTER")
    end

    function B.I.Main.Frame.Count:Hide()
        T.F.mainframe.search:Hide()
    end

    function B.I.Main.Frame.Count:Click(...)

        T.F.mainframe.bank:Hide()
        T.F.mainframe.reagent:Hide()
        T.F.mainframe.character:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.guild:Hide()

        if T.F.mainframe.search:IsVisible() then
            T.F.mainframe.search:Hide()
            if T.F.mainframe.lastbank then T.F.mainframe[T.F.mainframe.lastbank]:Show() end
        else
            T.F.mainframe.search:Show()
        end
    end

    B.I.Main.Search = {}
    function B.I.Main.Search.Create()

        T.F.mainframe.search = CreateFrame("Frame", "BWsearch", T.F.mainframe)
        T.F.mainframe.search:SetHeight(200)
        T.F.mainframe.search:SetPoint("TOPLEFT", T.F.mainframe.searchbar, "BOTTOMLEFT", -10, 0)
        T.F.mainframe.search:SetPoint("TOPRIGHT", T.F.mainframe.countbutton, "BOTTOMRIGHT")

        T.F.mainframe.search:Hide()
        T.F.mainframe.search:EnableMouse(true)

        T.F.mainframe.search.enddot = T.F.mainframe.search:CreateTexture(nil, "OVERLAY")
        T.F.mainframe.search.enddot:SetMask("Textures\\MinimapMask")
        T.F.mainframe.search.enddot:SetHeight(5)
        T.F.mainframe.search.enddot:SetWidth(5)
        T.F.mainframe.search.enddot:SetTexture("Interface\\Buttons\\WHITE8x8")
        T.F.mainframe.search.enddot:SetVertexColor(0.7, 0.7, 0.7, 1)
        T.F.mainframe.search.enddot:SetPoint("TOP", T.F.mainframe.search, "BOTTOM", 0, -5)

        B.I.Main.Search.Scroll()
    end

    function B.I.Main.Search.Scroll()

        T.F.mainframe.search.list = CreateFrame("Frame", "FauxScrollFrameBW", T.F.mainframe.search)
        T.F.mainframe.search.list:SetPoint("TOPLEFT", T.F.mainframe.search, "TOPLEFT", 0, -T.FRAMEWHITESPACE)
        T.F.mainframe.search.list:SetPoint("BOTTOMRIGHT", T.F.mainframe.search, "BOTTOMRIGHT")
        T.F.mainframe.search.list:SetWidth(T.F.mainframe.search:GetWidth())

        T.F.mainframe.search.scroll = B.I.ScrollFrame(T.F.mainframe.search.list, "BWScrollFrame",
                                                      B.ScrollbarUpdate, B.I.Main.Search.Button.Create)
    end

    B.I.Main.Search.Button = {}
    function B.I.Main.Search.Button.Create(index)

        local button = B.I.List.Button(index, T.F.mainframe.search.list)

        button:SetScript("OnEnter", B.I.Main.Search.Button.Enter)
        button:SetScript("OnLeave", B.I.Main.Search.Button.Leave)
        button:SetScript("OnClick", B.I.Main.Search.Button.Click)

        local leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        leftstring:SetJustifyH("LEFT")
        leftstring:SetJustifyV("CENTER")
        leftstring:SetWordWrap(false)
        leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        leftstring:SetPoint("RIGHT", button, "RIGHT", -T.F.mainframe.countbutton:GetWidth(), 0)
        button.leftstring = leftstring

       return button
    end

    function B.I.Main.Search.Button:Enter()
        if self.quest then
            self:SetBackdropBorderColor(1, 1, 0, 0.5)
        else
            local r, g, b = GetItemQualityColor(self.quality)
            self:SetBackdropBorderColor(r, g, b, 0.5)
        end
        B.I.Button.Enter(self)
    end

    function B.I.Main.Search.Button:Leave()
        self:SetBackdropBorderColor(0, 0, 0, 0.9)
        GameTooltip:Hide()
    end

    function B.I.Main.Search.Button:Click(...)
        local location, page, player

        location = self.location.loc
        if location ~= "alt" then
            page = tonumber(self.location.num)
        else
            player = self.location.num
        end

        if location == "bank" then
            T.F.mainframe.bankbutton:Click()
        elseif location == "reagent" then
            T.F.mainframe.reagentbutton:Click()
        elseif location == "void" then
            T.F.mainframe.voidbutton:Click()
            T.F.mainframe.void[page].tab:Click()
        elseif location == "guild" then
            T.F.mainframe.guildbutton:Click()
            T.F.mainframe.guild[page].tab:Click()
        elseif location == "player" then
            T.F.mainframe.characterbutton:Click()
        elseif location == "alt" then
            B.load(T.ACTIVE_TABLE.server, player, T.ACTIVE_TABLE.faction)
        elseif location == "equip" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.equipment:IsVisible() then T.F.mainframe.character.tab:Click() end
        end
    end

--[[//////////////////////////////////////////////////

    CREATE FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Button = {}
    function B.I.Button.Create(index, parent, splitstack)

        local button = CreateFrame("Button", nil, parent, "SecureActionButtonTemplate")

        button:SetID(index)
        button:SetHeight(T.BUTTON_HEIGHT)
        button:SetWidth(T.BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetPoint("CENTER")
        icon:SetHeight(T.BUTTON_HEIGHT - 3)
        icon:SetWidth(T.BUTTON_HEIGHT - 3)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button.icon = icon

        local search = button:CreateTexture(nil, "OVERLAY")
        search:SetAllPoints()
        search:SetTexture("Interface\\Buttons\\WHITE8x8")
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetAllPoints()
        highlight:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
        highlight:SetVertexColor(1, 1, 1, 1)
        button:SetHighlightTexture(highlight)

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(button.text:GetFont(), 14, "OUTLINE")
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT")
        button.text:SetPoint("BOTTOM", 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(button.ilvl:GetFont(), 12.5, "OUTLINE")
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", 1)
        button.ilvl:SetPoint("BOTTOM", 0, 1)

        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetAttribute("type", "macro")
        button:HookScript("PreClick", B.I.Button.PreClick)
        button:HookScript("OnClick", B.I.Button.Click)

        if splitstack then button.SplitStack = splitstack end

        return button
    end

    function B.I.Button:Update(elapsed)
        if not elapsed then return end

        if IsShiftKeyDown() and (IsModifiedClick("COMPAREITEMS") or GetCVarBool("alwaysCompareItems")) then
            GameTooltip_ShowCompareItem(GameTooltip)
            return
        end

        if GameTooltip.shoppingTooltips then -- Reset compares
            for _, frame in pairs(GameTooltip.shoppingTooltips) do
                frame:Hide()
            end
        end

        if IsControlKeyDown() then
            ShowInspectCursor()
        elseif T.ISMERCHANTOPEN and MerchantFrame.selectedTab == 1 and T.F.mainframe.lastbank == "character" and self.sell then
            SetCursor('BUY_CURSOR')
        else
            ResetCursor()
        end
    end

    function B.I.Button:PreClick(button)

        if not IsShiftKeyDown() and button == "RightButton" then

            if T.F.mainframe.lastbank == "character" then
                self:SetAttribute("macrotext", format("/use %d %d", B.GetBag(self)))
            elseif T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN then
                AutoStoreGuildBankItem(T.F.mainframe.guild.lasttab, self:GetID())
            elseif T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN then
                if not ReagentBankFrame:IsVisible() then BankFrameTab2:Click() end
                UseContainerItem(REAGENTBANK_CONTAINER, self:GetID())
            elseif T.F.mainframe.lastbank == "void" and T.ISVOIDOPEN then
                ClickVoidStorageSlot(T.F.mainframe.void.lasttab, self:GetID(), true)
            elseif T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN then
                if not BankFrameItem1:IsVisible() then BankFrameTab1:Click() end
                UseContainerItem(B.GetBankBag(self))
            end
        end
    end

    function B.I.Button:Click(button)

        if IsControlKeyDown() and button == "LeftButton" then
            DressUpItemLink(self.link)

        elseif IsShiftKeyDown() and button == "LeftButton" then

            if T.F.mainframe.searchbar:HasFocus() then
                T.F.mainframe.searchbar:SetText(self.name)
            elseif T.F.mainframe.lastbank == "character" and self.count > 1 then
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN and self.count > 1 then
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN and self.count > 1 then
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            elseif T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN and self.count > 1 then
                OpenStackSplitFrame(self.count, self, "BOTTOMLEFT", "TOPLEFT")
            else
                HandleModifiedItemClick(self.link)
            end

        elseif IsShiftKeyDown() and button == "RightButton" then

            if T.F.mainframe.lastbank == "character" and ( IsModifiedClick("SOCKETITEM") ) then
                if self:GetParent() == T.F.mainframe.character.equipment.frame then -- equipment tab
                    local id = self:GetID()
                    SocketInventoryItem(id == 18 and 19 or id)
                else
                    SocketContainerItem(B.GetBag(self))
                end
            end

        elseif button == "LeftButton" then

            if ( SpellCanTargetItem() or SpellCanTargetItemID() ) then
                -- Target the spell with the selected item
                if T.F.mainframe.lastbank == "character" then UseContainerItem(B.GetBag(self)); return end
            end
            
            if T.F.mainframe.lastbank == "character" then
                PickupContainerItem(B.GetBag(self))
            elseif T.F.mainframe.lastbank == "guild" and T.ISGUILDOPEN then
                PickupGuildBankItem(T.F.mainframe.guild.lasttab, self:GetID())
            elseif T.F.mainframe.lastbank == "bank" and T.ISBANKOPEN then
                if not BankFrameItem1:IsVisible() then BankFrameTab1:Click() end
                PickupContainerItem(B.GetBankBag(self))
            elseif T.F.mainframe.lastbank == "void" and T.ISVOIDOPEN then
                ClickVoidStorageSlot(T.F.mainframe.void.lasttab, self:GetID(), false)
            elseif T.F.mainframe.lastbank == "reagent" and T.ISBANKOPEN then
                if not ReagentBankFrame:IsVisible() then BankFrameTab2:Click() end
                PickupContainerItem(REAGENTBANK_CONTAINER, self:GetID())
            end

        elseif button == "RightButton" then

            self:SetAttribute("macrotext", nil)
        end
    end

    function B.I.Button:Enter(...)

        if self.link then
            self:SetScript("OnUpdate", B.I.Button.Update)
            GameTooltip:SetOwner(self, "ANCHOR_NONE")
            GameTooltip:ClearAllPoints()
            GameTooltip:SetPoint("TOPRIGHT", self, "TOPLEFT", -10, -20)

            if strmatch(self.link, "|Hbattlepet:") then
                -- Battle pet link
                local _, speciesID, level, breedQuality, maxHealth, power, speed, name = strsplit(":", self.link)
                BattlePetToolTip_Show(tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed), name)
                local r, g, b = GetItemQualityColor(tonumber(breedQuality))
                BattlePetTooltip:SetBackdropBorderColor(r, g, b)
            else
                GameTooltip:SetHyperlink(self.link)
                if self.quest then
                    GameTooltip:SetBackdropBorderColor(1, 1, 0)
                elseif self.quality then
                    local r, g, b = GetItemQualityColor(self.quality)
                    GameTooltip:SetBackdropBorderColor(r, g, b)
                end

                GameTooltip:Show()
            end
        end
    end

    function B.I.Button:Leave(...)
        ResetCursor()
        self:SetScript("OnUpdate", nil)
        GameTooltip_Hide() -- Blizzard function to close both gametooltip and battle pets
    end

    B.I.List = {}
    function B.I.List.Button(index, parent)

        local button = CreateFrame("Button", nil, parent)

        button:SetHeight(32)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 2})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 0.9)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetTexture("Interface\\Buttons\\WHITE8x8")
        highlight:SetVertexColor(0.7, 0.7, 0.7, 0.05)
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(28)
        icon:SetWidth(28)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(button.text:GetFont(), 12, "OUTLINE")
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(button.ilvl:GetFont(), 12, "OUTLINE")
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon, 1)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 1)

        button.rightstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.rightstring:SetJustifyH("CENTER")
        button.rightstring:SetJustifyV("CENTER")
        button.rightstring:SetPoint("RIGHT", -10, 0)
        button.rightstring:SetWidth(45)
        button.rightstring:SetWordWrap(false)
        button.rightstring:SetTextColor(1, 1, 1)

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(button.rightstring:GetWidth() + 15)
        button.rightstring.bg:SetTexture("Interface\\Buttons\\WHITE8x8")
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetWidth(button:GetWidth() - button.rightstring.bg:GetWidth())

        button.altstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        button.altstring:SetJustifyH("LEFT")
        button.altstring:SetJustifyV("CENTER")
        button.altstring:SetPoint("RIGHT", -10, 0)
        button.altstring:SetWordWrap(false)
        button.altstring:SetTextColor(1, 1, 1)

        return button
    end

    B.I.Tab = {}
    function B.I.Tab.Create(parent, name, inherit)

        local button = CreateFrame("Button", name, parent, inherit)
        button:SetHeight(32)
        button:SetWidth(33)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = "Interface\\Buttons\\WHITE8x8", edgeFile = "Interface\\Buttons\\WHITE8x8", edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local normal = button:CreateTexture(nil, "OVERLAY")

        normal:SetPoint("CENTER")
        normal:SetHeight(button:GetHeight() - 2)
        normal:SetWidth(button:GetWidth() - 2)
        normal:SetTexture("")
        normal:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button:SetNormalTexture(normal)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetAllPoints()
        highlight:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
        highlight:SetVertexColor(0.949, 0.847, 0.745, 1)
        button:SetHighlightTexture(highlight)

        local search = button:CreateTexture(nil, "OVERLAY")

        search:SetAllPoints(normal)
        search:SetTexture("Interface\\Buttons\\WHITE8x8")
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        return button
    end

    B.I.Frame = {}
    function B.I.Frame.Create(parent, pframe, index)
        -- Setup base for holding our buttons
        pframe.buttons = {}

        local frame = CreateFrame("Frame", nil, parent)
        frame:SetPoint("BOTTOM")
        frame:SetPoint("TOPLEFT", 10, -20)
        frame:SetFrameStrata("HIGH")

        if index then frame:SetID(index) end
        if index and index > 1 then frame:Hide() end

        pframe.frame = frame
    end

    B.I.Grid = {}
    function B.I.Grid.Create(parent, maxslots, rowcounter, numcounter, splitstack)
        local x, button, lastbutton, lastrow, rows, num

        rows = rowcounter or 0
        num = numcounter or 0

        for x = 1, maxslots do

            button = B.I.Button.Create(x, parent.frame, splitstack)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            
            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT", 1, 0)
                rows, num = rows + 1, 1
                lastrow = button

            elseif num == T.ROW then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end
            parent.buttons[x] = button
            lastbutton = button
        end

        return rows, num
    end

    function B.I.Grid.Add(parent, maxslots, rows, num, splitstack)
        local x, button, lastbutton, lastrow, numslots

        numslots = (T.ROW * (rows - 1)) + num
        lastbutton = parent.buttons[numslots]
        lastrow = parent.buttons[T.ROW * (rows - 1) + 1]

        for x = numslots + 1, maxslots do

            if parent.buttons[x] then
                parent.buttons[x]:Show()

                if num == T.ROW then
                    rows, num = rows + 1, 1
                else
                    num = num + 1
                end
                lastbutton = parent.buttons[x]
            else
                button = B.I.Button.Create(x, parent.frame, splitstack)
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)

                if num == T.ROW then -- Last column of the row
                    button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                    lastrow = button
                    rows, num = rows + 1, 1
                else
                    button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                    num = num + 1
                end
                parent.buttons[x] = button
                lastbutton = button
            end
        end

        return rows, num
    end

    function B.I.Grid.Remove(parent, maxslots, rows, num)
        local x, numslots, difference

        numslots = (T.ROW * (rows - 1)) + num
        difference = numslots - maxslots

        for x = numslots, (numslots - difference + 1), -1 do

            parent.buttons[x]:Hide()

            if num == 1 then
                rows, num = rows - 1, T.ROW
            else
                num = num - 1
            end
        end
        return rows, num
    end

    function B.I.Backdrop(frame)

        frame:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            tile = false, tileSize = 32, edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        frame:SetBackdropBorderColor(0, 0, 0, 1)
        frame:SetBackdropColor(0, 0, 0, 0.5)
        frame:EnableMouse(true)
    end

    B.I.GradientButton = {}
    function B.I.GradientButton.Create(button, orientation)

        button:SetNormalTexture("")
        button:SetHighlightTexture("")
        button:SetPushedTexture("")
        button:SetDisabledTexture("")

        B.I.RemoveEdge(button)

        button:SetBackdrop({bgFile = AS_backdrop,
                            edgeFile = AS_backdrop,
                            edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        button:SetBackdropColor(0, 0, 0, 0)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local gradient = button:CreateTexture(nil, "BACKGROUND")

        gradient:SetColorTexture(1, 1, 1)
        gradient:SetAllPoints()
        gradient:SetGradient(orientation, 0, 0, 0, 0.15, 0.15, 0.15)
    end

    function B.I.RemoveEdge(frame)

        local name = frame:GetName()
        local left = frame.Left or name and _G[name.."Left"] or nil
        local middle = frame.Middle or name and (_G[name.."Middle"] or _G[name.."Mid"]) or nil
        local right = frame.Right or name and _G[name.."Right"] or nil

        if left then left:Hide() end
        if middle then middle:Hide() end
        if right then right:Hide() end
    end

    function B.I.ScrollFrame(parent, name, onUpdate, buttons)

        local scrollframe = B.I.Scroll.Create(parent, name, onUpdate)
        local i, currentrow, previousrow

        parent.buttons = {}

        for i = 1, B.RowsToFit() do
            parent.buttons[i] = buttons(i)
            currentrow = parent.buttons[i]
            if i == 1 then
                currentrow:SetPoint("TOP")
            else
                currentrow:SetPoint("TOP", previousrow, "BOTTOM", 0, -1)
            end
            currentrow:Show()
            previousrow = currentrow
        end

        return scrollframe
    end

    B.I.Scroll = {}
    function B.I.Scroll.Create(parent, name, onUpdate)

        --[[note the anchors: the area of the scrollframe is the scrollable area
            that intercepts mousewheel to scroll. it does not include the scrollbar,
            which is anchored off the right ]]

        local scroll = CreateFrame("ScrollFrame", name, parent, "FauxScrollFrameTemplate")
        scroll:SetPoint("TOPLEFT", T.F.mainframe.search, "TOPLEFT")
        scroll:SetPoint("BOTTOMRIGHT", T.F.mainframe.search, "BOTTOMRIGHT")
        scroll.onUpdate = onUpdate

        scroll:SetScript("OnShow", onUpdate)
        scroll:SetScript("OnVerticalScroll", B.I.Scroll.VerticalScroll)
        scroll.ScrollBar:SetMinMaxValues(0, 100)
        scroll.ScrollBar:Hide()

        return scroll
    end

    function B.I.Scroll:VerticalScroll(offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, 32, self.onUpdate)
    end

    B.I.ColorPicker = {}
    function B.I.ColorPicker.Create(r, g, b, a, changedCallback)
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = nil, nil, nil
        ColorPickerFrame:SetColorRGB(r, g, b);
        ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
        ColorPickerFrame.previousValues = {r, g, b, a};
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = 
        changedCallback, changedCallback, changedCallback;
        ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
        ColorPickerFrame:Show();
    end

--[[//////////////////////////////////////////////////

    ANCHOR FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    function B.I.AnchorBagsTopLeft()
        -- DEFAULT LAYOUT
        T.F.mainframe.layout = 1
        B.AnchorFrames("TOP", "BOTTOM", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsBottomLeft()

        T.F.mainframe.layout = 2
        B.AnchorFrames("BOTTOM", "TOP", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsTopRight()

        T.F.mainframe.layout = 3
        B.AnchorFrames("TOP", "BOTTOM", "RIGHT", "LEFT")
    end

    function B.I.AnchorBagsBottomRight()

        T.F.mainframe.layout = 4
        B.AnchorFrames("BOTTOM", "TOP", "RIGHT", "LEFT")
    end

    function B.I.Main.Frame.ButtonHighlight(button, orientation)
        local highlight = button:GetHighlightTexture()

        highlight:ClearAllPoints()
        highlight:SetPoint(orientation.."LEFT", button, orientation.."LEFT")
        highlight:SetPoint(orientation.."RIGHT", button, orientation.."RIGHT")
        highlight:SetHeight(2)
    end

    function B.I.Main.Frame.ButtonOrder(orientation)

        T.F.mainframe.optionbutton:ClearAllPoints()
        T.F.mainframe.bankbutton:ClearAllPoints()
        T.F.mainframe.reagentbutton:ClearAllPoints()
        T.F.mainframe.characterbutton:ClearAllPoints()
        T.F.mainframe.voidbutton:ClearAllPoints()
        T.F.mainframe.guildbutton:ClearAllPoints()
        T.F.mainframe.searchbar:ClearAllPoints()
        T.F.mainframe.countbutton:ClearAllPoints()
        T.F.mainframe.search:ClearAllPoints()

        if orientation == "LEFT" then
            B.I.Main.Frame.ButtonLeft("LEFT", "RIGHT")
        else
            B.I.Main.Frame.ButtonRight("RIGHT", "LEFT")
        end
    end

    function B.I.Main.Frame.ButtonLeft(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.bankbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.guildbutton, direction, 10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.I.Main.Frame.ButtonRight(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.guildbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.bankbutton, direction, -10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.AnchorFrames(startpoint, endpoint, from, to)

        B.I.Main.Frame.ButtonOrder(from)

        BWdropDownMenu:ClearAllPoints()
        BWdropDownMenu.point = startpoint..from
        BWdropDownMenu.relativePoint = endpoint..from

        -- bank
        B.AnchorBank(startpoint, endpoint, from, to)
        -- reagent
        B.AnchorReagent(startpoint, endpoint, from, to)
        -- character
        B.AnchorCharacter(startpoint, endpoint, from, to)
        B.AnchorCharacterEquipment(startpoint, endpoint, from, to)
        -- void
        B.AnchorVoid(startpoint, endpoint, from, to)
        -- guild
        B.AnchorGuild(startpoint, endpoint, from, to)
        -- search
        B.AnchorSearchList(startpoint, endpoint, from, to)
    end

    function B.AnchorBank(startpoint, endpoint, from, to)

        T.F.mainframe.bank:ClearAllPoints()
        T.F.mainframe.bank:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.bankbutton, endpoint)

        for x = 1, NUM_BANKBAGSLOTS do
            T.F.mainframe.bank.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.bank.bags[x].tab:SetPoint("TOP"..from, T.F.mainframe.bank, "TOP"..to, 0, -1)
            else
                T.F.mainframe.bank.bags[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.bank.bags[x].tab
        end

        T.F.mainframe.bank.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.bank.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.bank.bagstoggle:ClearAllPoints()
        T.F.mainframe.bank.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.bank.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end

    function B.AnchorCharacter(startpoint, endpoint, from, to)

        T.F.mainframe.character:ClearAllPoints()
        T.F.mainframe.character:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.characterbutton, endpoint)

        T.F.mainframe.character.tab:ClearAllPoints()
        T.F.mainframe.character.tab:SetPoint("TOP"..from, T.F.mainframe.character, "TOP"..to, 0, -1)

        T.F.mainframe.character.artifact:ClearAllPoints()
        T.F.mainframe.character.artifact:SetPoint("TOP"..from, T.F.mainframe.character.tab, "BOTTOM"..from, 0, -2)

        for x = 1, NUM_BAG_SLOTS do
            T.F.mainframe.character.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.character.bags[x].tab:SetPoint("TOP"..from, T.F.mainframe.character.artifact, "BOTTOM"..from, 0, -2)
            else
                T.F.mainframe.character.bags[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -2)
            end
            lasttab = T.F.mainframe.character.bags[x].tab
        end

        T.F.mainframe.character.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.character.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.character.bagstoggle:ClearAllPoints()
        T.F.mainframe.character.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.character.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end

    function B.AnchorCharacterEquipment(startpoint, endpoint, from, to)

        T.F.mainframe.character.equipment:ClearAllPoints()
        T.F.mainframe.character.equipment:SetPoint("TOP"..from, T.F.mainframe.character.tab, "TOP"..to)
    end

    function B.AnchorReagent(startpoint, endpoint, from, to)

        T.F.mainframe.reagent:ClearAllPoints()
        T.F.mainframe.reagent:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.reagentbutton, endpoint)
    end

    function B.AnchorVoid(startpoint, endpoint, from, to)

        T.F.mainframe.void:ClearAllPoints()
        T.F.mainframe.void:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.voidbutton, endpoint)

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, T.F.mainframe.void, "TOP"..to, 0, -1)
            else
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.void[x].tab
        end
    end

    function B.AnchorGuild(startpoint, endpoint, from, to)

        T.F.mainframe.guild:ClearAllPoints()
        T.F.mainframe.guild:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.guildbutton, endpoint)

        for x = 1, MAX_GUILDBANK_TABS do
            T.F.mainframe.guild[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, T.F.mainframe.guild, "TOP"..to, 0, -1)
            else
                T.F.mainframe.guild[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            lasttab = T.F.mainframe.guild[x].tab
        end
    end

    function B.AnchorSearchList(startpoint, endpoint, from, to)

        T.F.mainframe.search:ClearAllPoints()
        T.F.mainframe.search:SetPoint(startpoint..from, T.F.mainframe.searchbar, endpoint..from, from == "LEFT" and -10 or 10, 0)
        T.F.mainframe.search:SetPoint(startpoint..to, T.F.mainframe.countbutton, endpoint..to)

        T.F.mainframe.search.enddot:ClearAllPoints()
        T.F.mainframe.search.enddot:SetPoint(startpoint, T.F.mainframe.search, endpoint, 0, startpoint == "TOP" and -5 or 5)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.countbutton, endpoint)

        T.F.mainframe.search.list:ClearAllPoints()
        T.F.mainframe.search.list:SetPoint(startpoint..from, T.F.mainframe.search, 0, startpoint == "TOP" and -T.FRAMEWHITESPACE or T.FRAMEWHITESPACE)
        T.F.mainframe.search.list:SetPoint(endpoint..to, T.F.mainframe.search)
    end
